*******************************************************************************
*                              Model: Scenario B                              *
*******************************************************************************

***Parameters declaration
PARAMETERS
                 supply03(i)                       Initial supply in i (in tons)
                 A(i)                              Inverse demand function intercepts p=f(q)
                 B(i)                              Absolute value of the inverse demand function slopes p=f(q)
                 C(i)                              Inverse supply function intercepts p=f(q)
                 D(i)                              Absolute value of the inverse supply function slopes p=f(q)
                 sup_cost(i,j)                     Parameter used to simulate an additional cost on trade from i to j (in USD per ton)    ;

***Parameters definition
supply03(i)   = sum[j,trade02(i,j)];
sup_cost(i,j) = 0

Parameter        Ed(i)                             demand price elasticity of maize
*We use a demand price elasticity of maize
/
KEN    0.148
TZA    0.078
UGA    0.002
ZMB    0.014
ZWE    0.001
/;


Parameter        Es(i)                             supply price elasticity of maize
*We use a supply price elasticity of maize
/
KEN    1.7
TZA    1.3
UGA    1.2
ZMB    1.5
ZWE    1.1
/;

* We then calculate the paramaters of the inverse demand function
B(i)$demand02(i)= consprice03(i)/(demand02(i)*Ed(i));
D(i)$supply03(i)= prodprice03(i)/(supply03(i)*Es(i));


*********The previous formula can only be used if demand and supply are positive.
*********For other countries (where demand or supply is not positive) we use the average of the slopes of the other functions.
Parameters countd, counts ;
countd=0;
counts=0;
loop(i,
         countd$demand02(i)=countd+1;
         counts$supply03(i)=counts+1
     ) ;

B(i)$(demand02(i)=0)= sum(j$demand02(j),B(j))/countd;
D(i)$(supply03(i)=0)= sum(j$supply03(j),D(j))/counts;

*We also create the parameter epsilon so that complementarity relationships are verified.
Parameter epsilon ;
epsilon=0.001;

A(i)= consprice03(i) + B(i)*demand02(i)-epsilon$(demand02(i)=0) ;
C(i)= prodprice03(i) - D(i)*supply03(i)+epsilon$(supply03(i)=0) ;

***Variables declaration
POSITIVE VARIABLES
                 demand(i)                         Quantity Demanded in i (in tons)
                 supply(i)                         Quantity Supplied in i (in tons)
                 consprice3(i)                     Market Demand Price in i (in USD per ton)
                 prodprice3(i)                     Market Supply Price in i (in USD per ton)
                 trade3(i,j)                       Shipments between regions (New matrix of the net trade from i to j (in tons)) ;

***Variables initialization
demand.l(i)               = demand02(i) ;
supply.l(i)               = supply03(i) ;
consprice3.l(i)           = consprice03(i);
prodprice3.l(i)           = prodprice03(i);
trade3.l(i,j)             = trade02(i,j);

***Equations declaration
EQUATIONS
                 Eq_PROD(i)                        Quantity produced greater than quantity shipped to all regions
                 Eq_DEM(i)                         Quantity demanded less than quantity shipped into region
                 Eq_DPRICEDIF(i)                   Difference between the market demand price and local demand price
                 Eq_SPRICEDIF(i)                   Difference between the market supply price and local supply price
                 PRLINK2(i,j)                      Price chain constraint 2      ;

***Equations definition
Eq_PROD(i)..           supply(i) =G= sum[j,trade3(i,j)] ;
Eq_DEM(i)..            sum[j,trade3(j,i)] =G= demand(i);
Eq_DPRICEDIF(i)..      consprice3(i) =G= A(i)- B(i)*demand(i) ;
Eq_SPRICEDIF(i)..      C(i) + D(i)*supply(i)=G= prodprice3(i);
PRLINK2(i,j)..         prodprice3(i)+ transportcost03(i,j)+ sup_cost(i,j)+sdt(i,j)$specific
                         + [(transportcost03(i,j)+ prodprice3(i) + sup_cost(i,j))*avt(i,j)]$advalorem =G= consprice3(j);

***Defines the model. A mixed complementarity problem (MCP) is used so each equation should be linked to a variable
MODEL SPATIAL Partial equilibrium model
/
Eq_PROD.prodprice3
Eq_DEM.consprice3
Eq_DPRICEDIF.demand
Eq_SPRICEDIF.supply
PRLINK2.trade3
/;

***Sets the memory space
SPATIAL.workspace = 125 ;

***Sets the number of decimal places to be included
OPTION DECIMALS=8;

***This keyword controls the number of cases written to the LST file for each named equation in a model.
OPTION limcol=25;
OPTION limrow=25;

***Solves the model without shock, using Mixed Complementarity Problem
SOLVE SPATIAL using MCP;

***Declaration of parameters to fix the values of some keys variables before the shock as initial values
PARAMETER
                 trade3_ini(i,j)                   Initial trade from i to j  (in tons)
                 totaltrade_ini                    Initial total quantity traded (in tons)
                 exports_ini(i)                    Initial quantity exported from i (in tons)
                 imports_ini(j)                    Initial quantity imported by j (in tons)
                 supply_ini(i)                     Initial supply (in tons)
                 demand_ini(i)                     Initial demand (in tons)
                 totalsupply_ini                   Initial total supply (in tons)
                 totaldemand_ini                   Initial total demand (in tons)
                 consprice3_ini(i)                 Initial market demand price (in USD per ton)
                 prodprice3_ini(i)                 Initial market supply price (in USD per ton)
                 CoS_ini(i)                        Initial consumer surplus  (in USD mlns)
                 PrS_ini(i)                        Initial producer surplus  (in USD mlns)
                 PR_ini(i)                         Initial public revenue  (in USD mlns)
                 welfare_ini(i)                    Initial welfare   (in USD mlns)
                 worldwelfare_ini                  Initial worldwelfare   (in USD mlns) ;

***Definition of the parameters mentioned above
trade3_ini(i,j)           = trade3.l(i,j) ;
totaltrade_ini            = sum[(i,j),trade3.l(i,j)];
exports_ini(i)            = sum[j$(ord(j) ne ord(i)), trade3.l(i,j)];
imports_ini(j)            = sum{i$[ord(i) ne ord(j)], trade3.l(i,j)};
supply_ini(i)             =  supply.l(i) ;
demand_ini(i)             = demand.l(i) ;
totalsupply_ini           = sum[i, supply.l(i)];
totaldemand_ini           = sum[i, demand.l(i)];
consprice3_ini(i)         = consprice3.l(i) ;
prodprice3_ini(i)         = prodprice3.l(i) ;
CoS_ini(i)                = B(i)*[demand.l(i)**2]/2/1000000;
PrS_ini(i)                = D(i)*[supply.l(i)**2]/2/1000000;
PR_ini(j)                 = sum{i,{sdt(i,j)$specific
                            + [(transportcost03(i,j)+ prodprice3.l(i)+ sup_cost(i,j))*avt(i,j)]$advalorem}*trade3.l(i,j)}/1000000 ;
welfare_ini(i)            = CoS_ini(i) + PrS_ini(i) + PR_ini(i) ;
worldwelfare_ini          = sum[i,welfare_ini(i)] ;

***Implementation of Scenario B: rise in transport costs on exports from Uganda to all its partners.
***No rise in transport costs within Uganda.

sup_cost('UGA',j)        = 50 ;
sup_cost('UGA','UGA')    = 0 ;


***Solving the model again, after the shock (Scenario B, using Mixed Complementarity Problem
SOLVE SPATIAL using mcp ;

***Declaration of the same key parameters, but this time, their values will depend on the after-shock values of the model variables rather than on the pre-shock
***values of the model variables. Hence they are not saved as initial values. This will later allow for comparisons between pre-shock and post-shock values of
***some key variables.
Parameters
                 totaltrade                        Total trade (in tons)
                 exports(i)                        Exports from i (in tons)
                 imports(j)                        Imports by j (in tons)
                 totalsupply                       Total supply (in tons)
                 totaldemand                       Total demand (in tons)
                 CoS(i)                            Consumer surplus (in USD mlns)
                 PrS(i)                            Producer surplus (in USD mlns)
                 PR(i)                             Public revenue  (in USD mlns)
                 welfare(i)                        Welfare  (in USD mlns)
                 worldwelfare                      Worldwelfare  (in USD mlns) ;

***Definition of the parameters mentioned above.
totaltrade       = sum[(i,j),trade3.l(i,j)];
exports(i)       = sum[j$(ord(j) ne ord(i)), trade3.l(i,j)];
imports(j)       = sum{i$[ord(i) ne ord(j)], trade3.l(i,j)};
totalsupply      = sum[i, supply.l(i)];
totaldemand      = sum[i, demand.l(i)];
CoS(i)           = B(i)*(demand.l(i)**2)/2/1000000;
PrS(i)           = D(i)*(supply.l(i)**2)/2/1000000;
PR(j)            = sum{i,{sdt(i,j)$specific
                    + [(transportcost03(i,j)+ prodprice3.l(i)+ sup_cost(i,j))*avt(i,j)]$advalorem}*trade3.l(i,j)}/1000000 ;
welfare(i)       = CoS(i) + PrS(i) + PR(i) ;
worldwelfare     = sum[i,welfare(i)] ;

***Declaration of variation parameters. Those parameters represent the variation between pre-shock and post-shock (Scenario B) values of key variables.
Parameters
                 var_trade(i,j)                    Variation in the quantity traded between i and j (expressed as a percentage)
                 var_totaltrade                    Variation in total trade (expressed as a percentage)
                 var_exports(i)                    Variation in the quantity exported from i (expressed as a percentage)
                 var_imports(j)                    Variation in the quantity imported by j (expressed as a percentage)
                 var_supply(i)                     Variation in the quantity supplied in i (expressed as a percentage)
                 var_demand(i)                     Variation in the quantity demanded in j (expressed as a percentage)
                 var_totalsupply                   Variation in total supply (expressed as a percentage)
                 var_totaldemand                   Variation in total demand (expressed as a percentage)
                 var_consprice3(i)                 Variation in the market demand price (expressed as a pourcentage)
                 var_prodprice3(i)                 Variation in the market supply price (expressed as a pourcentage)
                 var_CoS(i)                        Variation in consumer surplus  (expressed as a pourcentage)
                 var_PrS(i)                        Variation in producer surplus  (expressed as a pourcentage)
                 var_PR(i)                         Variation in public revenue   (expressed as a pourcentage)
                 var_welfare(i)                    Variation in welfare   (expressed as a pourcentage)
                 varworldwelfare                   Variation in worldwelfare  (expressed as a pourcentage) ;

***Definition of the parameters described above
var_trade(i,j)$trade3_ini(i,j)                                          = 100*[trade3.l(i,j) - trade3_ini(i,j)]/trade3_ini(i,j);
var_trade(i,j)$[(trade3_ini(i,j)=0) and (trade3.l(i,j)>0)]              = 100;
var_totaltrade$totaltrade_ini                                           = 100*[totaltrade - totaltrade_ini]/totaltrade_ini;
var_totaltrade$[(totaltrade_ini=0) and (totaltrade>0)]                  = 100;
var_exports(i)$exports_ini(i)                                           = 100*[exports(i) - exports_ini(i)]/exports_ini(i);
var_exports(i)$[(exports_ini(i)=0) and (exports(i)>0)]                  = 100;
var_imports(j)$imports_ini(j)                                           = 100*[imports(j) - imports_ini(j)]/imports_ini(j);
var_imports(j)$[(imports_ini(j)=0) and (imports(j)>0)]                  = 100;
var_supply(i)$supply_ini(i)                                             = 100*[supply.l(i) - supply_ini(i)]/supply_ini(i);
var_supply(i)$[(supply_ini(i)=0) and (supply.l(i)>0)]                   = 100;
var_demand(i)$demand_ini(i)                                             = 100*[demand.l(i) - demand_ini(i)]/demand_ini(i);
var_demand(i)$[(demand_ini(i)=0) and (demand.l(i)>0)]                   = 100;
var_totalsupply$totalsupply_ini                                         = 100*[totalsupply - totalsupply_ini]/totalsupply_ini;
var_totalsupply$[(totalsupply_ini=0) and (totalsupply>0)]               = 100;
var_totaldemand$totaldemand_ini                                         = 100*[totaldemand - totaldemand_ini]/totaldemand_ini;
var_totaldemand$[(totaldemand_ini=0) and (totaldemand>0)]               = 100;
var_consprice3(i)$consprice3_ini(i)                                     = 100*[consprice3.l(i) - consprice3_ini(i)]/consprice3_ini(i);
var_consprice3(i)$[(consprice3_ini(i)=0) and (consprice3.l(i)>0)]       = 100;
var_prodprice3(i)$prodprice3_ini(i)                                     = 100*[prodprice3.l(i) - prodprice3_ini(i)]/prodprice3_ini(i);
var_prodprice3(i)$[(prodprice3_ini(i)=0) and (prodprice3.l(i)>0)]       = 100;
var_CoS(i)$CoS_ini(i)                                                   = 100*[CoS(i) - CoS_ini(i)]/CoS_ini(i) ;
var_CoS(i)$[(CoS_ini(i)=0) and (CoS(i)>0)]                              = 100;
var_PrS(i)$PrS_ini(i)                                                   = 100*[PrS(i) - PrS_ini(i)]/PrS_ini(i) ;
var_PrS(i)$[(PrS_ini(i)=0) and (PrS(i)>0)]                              = 100;
var_PR(i)$PR_ini(i)                                                     = 100*[PR(i) - PR_ini(i)]/PR_ini(i) ;
var_PR(i)$[(PR_ini(i)=0) and (PR(i)>0)]                                 = 100;
var_welfare(i)$welfare_ini(i)                                           = 100*[welfare(i) - welfare_ini(i)]/welfare_ini(i) ;
var_welfare(i)$[(welfare_ini(i)=0) and (welfare(i)>0)]                  = 100;
varworldwelfare$worldwelfare_ini                                        = 100*[worldwelfare-worldwelfare_ini]/worldwelfare_ini ;
varworldwelfare$[(worldwelfare_ini=0) and (worldwelfare>0)]             = 100;
